@@ -11,6 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 from dataclasses import dataclass
 import subprocess
@@ -27,7 +28,7 @@ from pathlib import Path
 # and cross file currently), and also assists with the reading environment
 # variables.
 #
-# At this time there isn't an ironclad difference between this an other sources
+# At this time there isn't an ironclad difference between this and other sources
 # of state like `coredata`. But one rough guide is much what is in `coredata` is
 # the *output* of the configuration process: the final decisions after tests.
 # This, on the other hand has *inputs*. The config files are parsed, but
@@ -90,7 +91,7 @@ CPU_FAMILIES_64_BIT = [
 ]
 
 # Map from language identifiers to environment variables.
-ENV_VAR_PROG_MAP: T.Mapping[str, str] = {
+ENV_VAR_COMPILER_MAP: T.Mapping[str, str] = {
     # Compilers
     'c': 'CC',
     'cpp': 'CXX',
@@ -101,6 +102,7 @@ ENV_VAR_PROG_MAP: T.Mapping[str, str] = {
     'objcpp': 'OBJCXX',
     'rust': 'RUSTC',
     'vala': 'VALAC',
+    'nasm': 'NASM',
 
     # Linkers
     'c_ld': 'CC_LD',
@@ -110,20 +112,36 @@ ENV_VAR_PROG_MAP: T.Mapping[str, str] = {
     'objc_ld': 'OBJC_LD',
     'objcpp_ld': 'OBJCXX_LD',
     'rust_ld': 'RUSTC_LD',
+}
 
+# Map from utility names to environment variables.
+ENV_VAR_TOOL_MAP: T.Mapping[str, str] = {
     # Binutils
-    'strip': 'STRIP',
     'ar': 'AR',
+    'as': 'AS',
+    'ld': 'LD',
+    'nm': 'NM',
+    'objcopy': 'OBJCOPY',
+    'objdump': 'OBJDUMP',
+    'ranlib': 'RANLIB',
+    'readelf': 'READELF',
+    'size': 'SIZE',
+    'strings': 'STRINGS',
+    'strip': 'STRIP',
     'windres': 'WINDRES',
 
     # Other tools
     'cmake': 'CMAKE',
     'qmake': 'QMAKE',
     'pkgconfig': 'PKG_CONFIG',
+    'pkg-config': 'PKG_CONFIG',
     'make': 'MAKE',
     'vapigen': 'VAPIGEN',
+    'llvm-config': 'LLVM_CONFIG',
 }
 
+ENV_VAR_PROG_MAP = {**ENV_VAR_COMPILER_MAP, **ENV_VAR_TOOL_MAP}
+
 # Deprecated environment variables mapped from the new variable to the old one
 # Deprecated in 0.54.0
 DEPRECATED_ENV_PROG_MAP: T.Mapping[str, str] = {
@@ -149,7 +167,7 @@ class Properties:
         return language + '_stdlib' in self.properties
 
     # Some of get_stdlib, get_root, get_sys_root are wider than is actually
-    # true, but without heterogenious dict annotations it's not practical to
+    # true, but without heterogeneous dict annotations it's not practical to
     # narrow them
     def get_stdlib(self, language: str) -> T.Union[str, T.List[str]]:
         stdlib = self.properties[language + '_stdlib']
@@ -429,7 +447,8 @@ class CMakeVariables:
         for key, value in variables.items():
             value = mesonlib.listify(value)
             for i in value:
-                assert isinstance(i, str)
+                if not isinstance(i, str):
+                    raise EnvironmentException(f"Value '{i}' of CMake variable '{key}' defined in a machine file is a {type(i).__name__} and not a str")
             self.variables[key] = value
 
     def get_variables(self) -> T.Dict[str, T.List[str]]:
