@@ -1,6 +1,7 @@
 # SPDX-License-Identifier: Apache-2.0
 # Copyright © 2021 The Meson Developers
 # Copyright © 2021 Intel Corporation
+from __future__ import annotations
 
 """Keyword Argument type annotations."""
 
@@ -56,7 +57,7 @@ class FuncTest(FuncBenchmark):
 
     """Keyword Arguments for `test`
 
-    `test` only adds the `is_prallel` argument over benchmark, so inherintance
+    `test` only adds the `is_parallel` argument over benchmark, so inheritance
     is helpful here.
     """
 
