@@ -13,7 +13,6 @@ from ..build import (CustomTarget, BuildTarget,
                      BothLibraries, SharedLibrary, StaticLibrary, Jar, Executable)
 from ..coredata import UserFeatureOption
 from ..dependencies import Dependency, InternalDependency
-from ..interpreterbase import FeatureNew
 from ..interpreterbase.decorators import KwargInfo, ContainerTypeInfo
 from ..mesonlib import (File, FileMode, MachineChoice, listify, has_path_sep,
                         OptionKey, EnvironmentVariables)
@@ -26,7 +25,6 @@ if T.TYPE_CHECKING:
     from typing_extensions import Literal
 
     from ..interpreterbase import TYPE_var
-    from ..interpreterbase.decorators import FeatureCheckBase
     from ..mesonlib import EnvInitValueType
 
     _FullEnvInitValueType = T.Union[EnvironmentVariables, T.List[str], T.List[T.List[str]], EnvInitValueType, str, None]
@@ -89,9 +87,9 @@ def _install_mode_validator(mode: T.List[T.Union[str, bool, int]]) -> T.Optional
             return f'permission character 9 must be "-", "t", "T", or "x", not {perms[8]}'
 
         if len(mode) >= 2 and not isinstance(mode[1], (int, str, bool)):
-            return 'second componenent can only be a string, number, or False'
+            return 'second component can only be a string, number, or False'
         if len(mode) >= 3 and not isinstance(mode[2], (int, str, bool)):
-            return 'third componenent can only be a string, number, or False'
+            return 'third component can only be a string, number, or False'
 
     return None
 
@@ -213,7 +211,7 @@ def _env_validator(value: T.Union[EnvironmentVariables, T.List['TYPE_var'], T.Di
     return None
 
 def _options_validator(value: T.Union[EnvironmentVariables, T.List['TYPE_var'], T.Dict[str, 'TYPE_var'], str, None]) -> T.Optional[str]:
-    # Reusing the env validator is a littl overkill, but nicer than duplicating the code
+    # Reusing the env validator is a little overkill, but nicer than duplicating the code
     return _env_validator(value, allow_dict_list=False)
 
 def split_equal_string(input: str) -> T.Tuple[str, str]:
@@ -298,6 +296,13 @@ OVERRIDE_OPTIONS_KW: KwargInfo[T.List[str]] = KwargInfo(
 
 
 def _output_validator(outputs: T.List[str]) -> T.Optional[str]:
+    output_set = set(outputs)
+    if len(output_set) != len(outputs):
+        seen = set()
+        for el in outputs:
+            if el in seen:
+                return f"contains {el!r} multiple times, but no duplicates are allowed."
+            seen.add(el)
     for i in outputs:
         if i == '':
             return 'Output must not be empty.'
@@ -376,13 +381,6 @@ INCLUDE_DIRECTORIES: KwargInfo[T.List[T.Union[str, IncludeDirs]]] = KwargInfo(
     default=[],
 )
 
-def include_dir_string_new(val: T.List[T.Union[str, IncludeDirs]]) -> T.Iterable[FeatureCheckBase]:
-    strs = [v for v in val if isinstance(v, str)]
-    if strs:
-        str_msg = ", ".join(f"'{s}'" for s in strs)
-        yield FeatureNew('include_directories kwarg of type string', '1.0.0',
-                         f'Use include_directories({str_msg}) instead')
-
 # for cases like default_options and override_options
 DEFAULT_OPTIONS: KwargInfo[T.List[str]] = KwargInfo(
     'default_options',
