@@ -11,6 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import abc
 import enum
@@ -66,7 +67,7 @@ class StaticLinker:
         return []
 
     def get_output_args(self, target: str) -> T.List[str]:
-        return[]
+        return []
 
     def get_coverage_link_args(self) -> T.List[str]:
         return []
@@ -137,12 +138,12 @@ class VisualStudioLikeLinker:
 
     @classmethod
     def unix_args_to_native(cls, args: T.List[str]) -> T.List[str]:
-        from ..compilers import VisualStudioCCompiler
+        from ..compilers.c import VisualStudioCCompiler
         return VisualStudioCCompiler.unix_args_to_native(args)
 
     @classmethod
     def native_args_to_unix(cls, args: T.List[str]) -> T.List[str]:
-        from ..compilers import VisualStudioCCompiler
+        from ..compilers.c import VisualStudioCCompiler
         return VisualStudioCCompiler.native_args_to_unix(args)
 
     def rsp_file_syntax(self) -> RSPFileSyntax:
@@ -208,8 +209,12 @@ class ArLinker(ArLikeLinker):
         return self.can_rsp
 
     def get_std_link_args(self, env: 'Environment', is_thin: bool) -> T.List[str]:
-        # FIXME: osx ld rejects this: "file built for unknown-unsupported file format"
-        if is_thin and not env.machines[self.for_machine].is_darwin():
+        # Thin archives are a GNU extension not supported by the system linkers
+        # on Mac OS X, Solaris, or illumos, so don't build them on those OSes.
+        # OS X ld rejects with: "file built for unknown-unsupported file format"
+        # illumos/Solaris ld rejects with: "unknown file type"
+        if is_thin and not env.machines[self.for_machine].is_darwin() \
+          and not env.machines[self.for_machine].is_sunos():
             return self.std_thin_args
         else:
             return self.std_args
@@ -456,6 +461,9 @@ class DynamicLinker(metaclass=abc.ABCMeta):
     def get_lto_args(self) -> T.List[str]:
         return []
 
+    def get_thinlto_cache_args(self, path: str) -> T.List[str]:
+        return []
+
     def sanitizer_args(self, value: str) -> T.List[str]:
         return []
 
@@ -580,6 +588,18 @@ class GnuLikeDynamicLinkerMixin:
         'custom': [],
     }  # type: T.Dict[str, T.List[str]]
 
+    _SUBSYSTEMS = {
+        "native": "1",
+        "windows": "windows",
+        "console": "console",
+        "posix": "7",
+        "efi_application": "10",
+        "efi_boot_service_driver": "11",
+        "efi_runtime_driver": "12",
+        "efi_rom": "13",
+        "boot_application": "16",
+    }  # type: T.Dict[str, str]
+
     def get_buildtype_args(self, buildtype: str) -> T.List[str]:
         # We can override these in children by just overriding the
         # _BUILDTYPE_ARGS value.
@@ -713,21 +733,10 @@ class GnuLikeDynamicLinkerMixin:
         # as well, and is always accepted, so we manually map the
         # other types here. List of all types:
         # https://github.com/wine-mirror/wine/blob/3ded60bd1654dc689d24a23305f4a93acce3a6f2/include/winnt.h#L2492-L2507
-        subsystems = {
-            "native": "1",
-            "windows": "windows",
-            "console": "console",
-            "posix": "7",
-            "efi_application": "10",
-            "efi_boot_service_driver": "11",
-            "efi_runtime_driver": "12",
-            "efi_rom": "13",
-            "boot_application": "16",
-        }
         versionsuffix = None
         if ',' in value:
             value, versionsuffix = value.split(',', 1)
-        newvalue = subsystems.get(value)
+        newvalue = self._SUBSYSTEMS.get(value)
         if newvalue is not None:
             if versionsuffix is not None:
                 newvalue += f':{versionsuffix}'
@@ -813,6 +822,14 @@ class AppleDynamicLinker(PosixDynamicLinkerMixin, DynamicLinker):
 
         return (args, set())
 
+    def get_thinlto_cache_args(self, path: str) -> T.List[str]:
+        return ["-Wl,-cache_path_lto," + path]
+
+
+class LLVMLD64DynamicLinker(AppleDynamicLinker):
+
+    id = 'ld64.lld'
+
 
 class GnuDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, DynamicLinker):
 
@@ -826,6 +843,9 @@ class GnuGoldDynamicLinker(GnuDynamicLinker):
 
     id = 'ld.gold'
 
+    def get_thinlto_cache_args(self, path: str) -> T.List[str]:
+        return ['-Wl,-plugin-opt,cache-dir=' + path]
+
 
 class GnuBFDDynamicLinker(GnuDynamicLinker):
 
@@ -836,6 +856,9 @@ class MoldDynamicLinker(GnuDynamicLinker):
 
     id = 'ld.mold'
 
+    def get_thinlto_cache_args(self, path: str) -> T.List[str]:
+        return ['-Wl,--thinlto-cache-dir=' + path]
+
 
 class LLVMDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, DynamicLinker):
 
@@ -853,7 +876,7 @@ class LLVMDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, Dyna
         super().__init__(exelist, for_machine, prefix_arg, always_args, version=version)
 
         # Some targets don't seem to support this argument (windows, wasm, ...)
-        _, _, e = mesonlib.Popen_safe(self.exelist + self._apply_prefix('--allow-shlib-undefined'))
+        _, _, e = mesonlib.Popen_safe(self.exelist + always_args + self._apply_prefix('--allow-shlib-undefined'))
         # Versions < 9 do not have a quoted argument
         self.has_allow_shlib_undefined = ('unknown argument: --allow-shlib-undefined' not in e) and ("unknown argument: '--allow-shlib-undefined'" not in e)
 
@@ -862,6 +885,21 @@ class LLVMDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, Dyna
             return self._apply_prefix('--allow-shlib-undefined')
         return []
 
+    def get_thinlto_cache_args(self, path: str) -> T.List[str]:
+        return ['-Wl,--thinlto-cache-dir=' + path]
+
+    def get_win_subsystem_args(self, value: str) -> T.List[str]:
+        # lld does not support a numeric subsystem value
+        version = None
+        if ',' in value:
+            value, version = value.split(',', 1)
+        if value in self._SUBSYSTEMS:
+            if version is not None:
+                value += f':{version}'
+            return self._apply_prefix([f'--subsystem,{value}'])
+        else:
+            raise mesonlib.MesonBugException(f'win_subsystem: {value} not handled in lld linker. This should not be possible.')
+
 
 class WASMDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, DynamicLinker):
 
@@ -870,10 +908,10 @@ class WASMDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, Dyna
     id = 'ld.wasm'
 
     def get_allow_undefined_args(self) -> T.List[str]:
-        return ['-s', 'ERROR_ON_UNDEFINED_SYMBOLS=0']
+        return ['-sERROR_ON_UNDEFINED_SYMBOLS=0']
 
     def no_undefined_args(self) -> T.List[str]:
-        return ['-s', 'ERROR_ON_UNDEFINED_SYMBOLS=1']
+        return ['-sERROR_ON_UNDEFINED_SYMBOLS=1']
 
     def get_soname_args(self, env: 'Environment', prefix: str, shlib_name: str,
                         suffix: str, soversion: str, darwin_versions: T.Tuple[str, str]) -> T.List[str]:
@@ -890,7 +928,7 @@ class WASMDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, Dyna
 
 class CcrxDynamicLinker(DynamicLinker):
 
-    """Linker for Renesis CCrx compiler."""
+    """Linker for Renesas CCrx compiler."""
 
     id = 'rlink'
 
@@ -1126,7 +1164,7 @@ class NAGDynamicLinker(PosixDynamicLinkerMixin, DynamicLinker):
         return []
 
     def get_std_shared_lib_args(self) -> T.List[str]:
-        from ..compilers import NAGFortranCompiler
+        from ..compilers.fortran import NAGFortranCompiler
         return NAGFortranCompiler.get_nagfor_quiet(self.version) + ['-Wl,-shared']
 
 
@@ -1178,7 +1216,7 @@ NvidiaHPC_StaticLinker = PGIStaticLinker
 
 class VisualStudioLikeLinkerMixin:
 
-    """Mixin class for for dynamic linkers that act like Microsoft's link.exe."""
+    """Mixin class for dynamic linkers that act like Microsoft's link.exe."""
 
     if T.TYPE_CHECKING:
         for_machine = MachineChoice.HOST
@@ -1304,6 +1342,9 @@ class ClangClDynamicLinker(VisualStudioLikeLinkerMixin, DynamicLinker):
     def get_win_subsystem_args(self, value: str) -> T.List[str]:
         return self._apply_prefix([f'/SUBSYSTEM:{value.upper()}'])
 
+    def get_thinlto_cache_args(self, path: str) -> T.List[str]:
+        return ["/lldltocache:" + path]
+
 
 class XilinkDynamicLinker(VisualStudioLikeLinkerMixin, DynamicLinker):
 
@@ -1405,6 +1446,12 @@ class AIXDynamicLinker(PosixDynamicLinkerMixin, DynamicLinker):
     def get_allow_undefined_args(self) -> T.List[str]:
         return self._apply_prefix(['-berok'])
 
+    def get_link_whole_for(self, args: T.List[str]) -> T.List[str]:
+        # AIX's linker always links the whole archive: "The ld command
+        # processes all input files in the same manner, whether they are
+        # archives or not."
+        return args
+
     def build_rpath_args(self, env: 'Environment', build_dir: str, from_dir: str,
                          rpath_paths: T.Tuple[str, ...], build_rpath: str,
                          install_rpath: str) -> T.Tuple[T.List[str], T.Set[bytes]]:
@@ -1442,7 +1489,7 @@ class OptlinkDynamicLinker(VisualStudioLikeLinkerMixin, DynamicLinker):
 
     def __init__(self, exelist: T.List[str], for_machine: mesonlib.MachineChoice,
                  *, version: str = 'unknown version'):
-        # Use optlink instead of link so we don't interfer with other link.exe
+        # Use optlink instead of link so we don't interfere with other link.exe
         # implementations.
         super().__init__(exelist, for_machine, '', [], version=version)
 
@@ -1475,7 +1522,7 @@ class CudaLinker(PosixDynamicLinkerMixin, DynamicLinker):
         # Built on Sun_Sep_30_21:09:22_CDT_2018
         # Cuda compilation tools, release 10.0, V10.0.166
         # we need the most verbose version output. Luckily starting with V
-        return out.strip().split('V')[-1]
+        return out.strip().rsplit('V', maxsplit=1)[-1]
 
     def get_accepts_rsp(self) -> bool:
         # nvcc does not support response files
@@ -1495,7 +1542,7 @@ class CudaLinker(PosixDynamicLinkerMixin, DynamicLinker):
         #
         #   nvcc fatal : Don't know what to do with 'subprojects/foo/libbar.so.0.1.2'
         #
-        from ..compilers import CudaCompiler
+        from ..compilers.cuda import CudaCompiler
         return CudaCompiler.LINKER_PREFIX
 
     def fatal_warnings(self) -> T.List[str]:
